searchData={"items":[{"type":"module","title":"PhoenixTest.Case","doc":"ExUnit case module to assist with browser based tests.\nSee `PhoenixTest.Playwright` for more information.","ref":"PhoenixTest.Case.html"},{"type":"module","title":"PhoenixTest.Playwright","doc":"> #### Warning {: .warning}\n>\n> This driver is experimental.\n> If you don't need browser based tests, see `m:PhoenixTest#module-usage` on regular usage.\n\nExecute PhoenixTest cases in an actual browser via [Playwright](https://playwright.dev/).","ref":"PhoenixTest.Playwright.html"},{"type":"module","title":"Example - PhoenixTest.Playwright","doc":"Refer to the accompanying example repo for a full example:\nhttps://github.com/ftes/phoenix_test_playwright_example/commits/main","ref":"PhoenixTest.Playwright.html#module-example"},{"type":"module","title":"Setup - PhoenixTest.Playwright","doc":"1. Add to `mix.exs` deps: `{:phoenix_test_playwright, \"~> 0.1\", only: :test, runtime: false}`\n2. Install Playwright: `npm --prefix assets i -D playwright`\n3. Install browsers: `npm --prefix assets exec playwright install --with-deps`\n4. Add to `config/test.exs`: `config :phoenix_test, otp_app: :your_app, playwright: [cli: \"assets/node_modules/playwright/cli.js\"]`\n5. Add to `config/test.exs`: `config :your_app, YourAppWeb.Endpoint, server: true`\n6. Add to `test/test_helpers.exs`: `Application.put_env(:phoenix_test, :base_url, YourAppWeb.Endpoint.url())`","ref":"PhoenixTest.Playwright.html#module-setup"},{"type":"module","title":"Usage - PhoenixTest.Playwright","doc":"```elixir\ndefmodule MyFeatureTest do\n  use PhoenixTest.Case, async: true\n  @moduletag :playwright\n\n  @tag trace: :open\n  test \"heading\", %{conn: conn} do\n    conn\n    |> visit(\"/\")\n    |> assert_has(\"h1\", text: \"Heading\")\n  end\nend\n```\n\nAs shown above, you can use `m:ExUnit.Case#module-parameterized-tests` parameterized tests\nto run tests concurrently in different browsers.","ref":"PhoenixTest.Playwright.html#module-usage"},{"type":"module","title":"Configuration - PhoenixTest.Playwright","doc":"In `config/test.exs`:\n\n```elixir\nconfig :phoenix_test,\n  otp_app: :your_app,\n  playwright: [\n    cli: \"assets/node_modules/playwright/cli.js\",\n    browser: [browser: :chromium, headless: System.get_env(\"PLAYWRIGHT_HEADLESS\", \"t\") in ~w(t true)],\n    trace: System.get_env(\"PLAYWRIGHT_TRACE\", \"false\") in ~w(t true),\n    trace_dir: \"tmp\"\n  ],\n  timeout_ms: 2000\n```","ref":"PhoenixTest.Playwright.html#module-configuration"},{"type":"module","title":"Playwright Traces - PhoenixTest.Playwright","doc":"You can enable [trace](https://playwright.dev/docs/trace-viewer-intro) recording in different ways:\n- Environment variable, see [Configuration](#module-configuration)\n- ExUnit `@tag :trace`\n- ExUnit `@tag trace: :open` to open the trace viewer automatically after completion","ref":"PhoenixTest.Playwright.html#module-playwright-traces"},{"type":"module","title":"Common problems - PhoenixTest.Playwright","doc":"- Test failures in CI (timeouts): Try less concurrency, e.g. `mix test --max-cases 1` for GitHub CI shared runners\n- LiveView not connected: add `assert_has(\"body .phx-connected\")` to test after `visit`ing (or otherwise navigating to) a LiveView\n- LiveComponent not connected: add `data-connected={connected?(@socket)}` to template and `assert_has(\"#my-component[data-connected]\")` to test","ref":"PhoenixTest.Playwright.html#module-common-problems"},{"type":"module","title":"Ecto SQL.Sandbox - PhoenixTest.Playwright","doc":"`PhoenixTest.Case` automatically takes care of this.\nIt passes a user agent referencing your Ecto repos.\nThis allows for [concurrent browser tests](https://hexdocs.pm/phoenix_ecto/main.html#concurrent-browser-tests).\n\nMake sure to follow the advanced set up instructions if necessary:\n- [with LiveViews](https://hexdocs.pm/phoenix_ecto/Phoenix.Ecto.SQL.Sandbox.html#module-acceptance-tests-with-liveviews)\n- [with Channels](https://hexdocs.pm/phoenix_ecto/Phoenix.Ecto.SQL.Sandbox.html#module-acceptance-tests-with-channels)\n\n```elixir\ndefmodule MyTest do\n  use PhoenixTest.Case, async: true\n```","ref":"PhoenixTest.Playwright.html#module-ecto-sql-sandbox"},{"type":"module","title":"Advanced assertions - PhoenixTest.Playwright","doc":"```elixir\ndef assert_has_value(session, label, value, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: Selector.label(label, opts),\n    expression: \"to.have.value\",\n    expectedText: [%{string: value}]\n  )\nend\n\ndef assert_has_selected(session, label, value, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: label |> Selector.label(opts) |> Selector.concat(\"option[selected]\"),\n    expression: \"to.have.text\",\n    expectedText: [%{string: value}]\n  )\nend\n\ndef assert_is_chosen(session, label, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: Selector.label(label, opts),\n    expression: \"to.have.attribute\",\n    expressionArg: \"checked\"\n  )\nend\n\ndef assert_is_editable(session, label, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: Selector.label(label, opts),\n    expression: \"to.be.editable\"\n  )\nend\n\ndef refute_is_editable(session, label, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(\n    session,\n    [\n      selector: Selector.label(label, opts),\n      expression: \"to.be.editable\"\n    ],\n    is_not: true\n  )\nend\n\ndef assert_found(session, params, opts \\\\ []) do\n  is_not = Keyword.get(opts, :is_not, false)\n  params = Enum.into(params, %{isNot: is_not})\n\n  unwrap(session, fn frame_id ->\n    {:ok, found} = Frame.expect(frame_id, params)\n    if is_not, do: refute(found), else: assert(found)\n  end)\nend\n\ndef assert_download(session, name, contains: content) do\n  assert_receive({:playwright, %{method: \"download\"} = download_msg}, 2000)\n  artifact_guid = download_msg.params.artifact.guid\n  assert_receive({:playwright, %{method: \"__create__\", params: %{guid: ^artifact_guid}} = artifact_msg}, 2000)\n  download_path = artifact_msg.params.initializer.absolutePath\n  wait_for_file(download_path)\n\n  assert download_msg.params.suggestedFilename =~ name\n  assert File.read!(download_path) =~ content\n\n  session\nend\n\ndefp wait_for_file(path, remaining_ms \\\\ 2000, wait_for_ms \\\\ 100)\ndefp wait_for_file(path, remaining_ms, _) when remaining_ms <= 0, do: flunk(\"File #{path} does not exist\")\n\ndefp wait_for_file(path, remaining_ms, wait_for_ms) do\n  if File.exists?(path) do\n    :ok\n  else\n    Process.sleep(wait_for_ms)\n    wait_for_file(path, remaining_ms - wait_for_ms, wait_for_ms)\n  end\nend\n```","ref":"PhoenixTest.Playwright.html#module-advanced-assertions"},{"type":"function","title":"PhoenixTest.Playwright.assert_has/2","doc":"","ref":"PhoenixTest.Playwright.html#assert_has/2"},{"type":"function","title":"PhoenixTest.Playwright.assert_has/3","doc":"","ref":"PhoenixTest.Playwright.html#assert_has/3"},{"type":"function","title":"PhoenixTest.Playwright.build/3","doc":"","ref":"PhoenixTest.Playwright.html#build/3"},{"type":"function","title":"PhoenixTest.Playwright.check/4","doc":"","ref":"PhoenixTest.Playwright.html#check/4"},{"type":"function","title":"PhoenixTest.Playwright.choose/4","doc":"","ref":"PhoenixTest.Playwright.html#choose/4"},{"type":"function","title":"PhoenixTest.Playwright.click/2","doc":"","ref":"PhoenixTest.Playwright.html#click/2"},{"type":"function","title":"PhoenixTest.Playwright.click/4","doc":"","ref":"PhoenixTest.Playwright.html#click/4"},{"type":"function","title":"PhoenixTest.Playwright.click_button/4","doc":"","ref":"PhoenixTest.Playwright.html#click_button/4"},{"type":"function","title":"PhoenixTest.Playwright.click_link/4","doc":"","ref":"PhoenixTest.Playwright.html#click_link/4"},{"type":"function","title":"PhoenixTest.Playwright.current_path/1","doc":"","ref":"PhoenixTest.Playwright.html#current_path/1"},{"type":"function","title":"PhoenixTest.Playwright.fill_in/4","doc":"","ref":"PhoenixTest.Playwright.html#fill_in/4"},{"type":"function","title":"PhoenixTest.Playwright.open_browser/2","doc":"","ref":"PhoenixTest.Playwright.html#open_browser/2"},{"type":"function","title":"PhoenixTest.Playwright.refute_has/2","doc":"","ref":"PhoenixTest.Playwright.html#refute_has/2"},{"type":"function","title":"PhoenixTest.Playwright.refute_has/3","doc":"","ref":"PhoenixTest.Playwright.html#refute_has/3"},{"type":"function","title":"PhoenixTest.Playwright.render_html/1","doc":"","ref":"PhoenixTest.Playwright.html#render_html/1"},{"type":"function","title":"PhoenixTest.Playwright.render_page_title/1","doc":"","ref":"PhoenixTest.Playwright.html#render_page_title/1"},{"type":"function","title":"PhoenixTest.Playwright.retry/2","doc":"","ref":"PhoenixTest.Playwright.html#retry/2"},{"type":"function","title":"PhoenixTest.Playwright.select/4","doc":"","ref":"PhoenixTest.Playwright.html#select/4"},{"type":"function","title":"PhoenixTest.Playwright.submit/1","doc":"","ref":"PhoenixTest.Playwright.html#submit/1"},{"type":"function","title":"PhoenixTest.Playwright.uncheck/4","doc":"","ref":"PhoenixTest.Playwright.html#uncheck/4"},{"type":"function","title":"PhoenixTest.Playwright.unwrap/2","doc":"","ref":"PhoenixTest.Playwright.html#unwrap/2"},{"type":"function","title":"PhoenixTest.Playwright.upload/5","doc":"","ref":"PhoenixTest.Playwright.html#upload/5"},{"type":"function","title":"PhoenixTest.Playwright.visit/2","doc":"","ref":"PhoenixTest.Playwright.html#visit/2"},{"type":"function","title":"PhoenixTest.Playwright.within/3","doc":"","ref":"PhoenixTest.Playwright.html#within/3"},{"type":"module","title":"PhoenixTest.Playwright.Browser","doc":"Interact with a Playwright `Browser`.\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/browser.ts","ref":"PhoenixTest.Playwright.Browser.html"},{"type":"function","title":"PhoenixTest.Playwright.Browser.new_context/2","doc":"Start a new browser context and return its `guid`.","ref":"PhoenixTest.Playwright.Browser.html#new_context/2"},{"type":"module","title":"PhoenixTest.Playwright.BrowserContext","doc":"Interact with a Playwright `BrowserContext`.\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/browserContext.ts","ref":"PhoenixTest.Playwright.BrowserContext.html"},{"type":"function","title":"PhoenixTest.Playwright.BrowserContext.new_page/1","doc":"Open a new browser page and return its `guid`.","ref":"PhoenixTest.Playwright.BrowserContext.html#new_page/1"},{"type":"function","title":"PhoenixTest.Playwright.BrowserContext.start_tracing/2","doc":"Start tracing. The results can be retrieved via `stop_tracing/2`.","ref":"PhoenixTest.Playwright.BrowserContext.html#start_tracing/2"},{"type":"function","title":"PhoenixTest.Playwright.BrowserContext.stop_tracing/2","doc":"Stop tracing and write zip file to specified output path.\n\nTrace can be viewed via either\n- `npx playwright show-trace trace.zip`\n- https://trace.playwright.dev","ref":"PhoenixTest.Playwright.BrowserContext.html#stop_tracing/2"},{"type":"module","title":"PhoenixTest.Playwright.Connection","doc":"Stateful, `GenServer` based connection to a Playwright node.js server.\nThe connection is established via `Playwright.Port`.\n\nYou won't usually have to use this module directly.\n`PhoenixTest.Case` uses this under the hood.","ref":"PhoenixTest.Playwright.Connection.html"},{"type":"function","title":"PhoenixTest.Playwright.Connection.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PhoenixTest.Playwright.Connection.html#child_spec/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.ensure_started/1","doc":"Lazy launch. Only start the playwright server if actually needed by a test.","ref":"PhoenixTest.Playwright.Connection.html#ensure_started/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.initializer/1","doc":"Get the initializer data for a channel.","ref":"PhoenixTest.Playwright.Connection.html#initializer/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.launch_browser/2","doc":"Launch a browser and return its `guid`.","ref":"PhoenixTest.Playwright.Connection.html#launch_browser/2"},{"type":"function","title":"PhoenixTest.Playwright.Connection.post/1","doc":"Post a message and await the response.\nWe wait for an additional grace period after the timeout that we pass to playwright.","ref":"PhoenixTest.Playwright.Connection.html#post/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.received/1","doc":"Get all past received messages for a playwright `guid` (e.g. a `Frame`).\nThe internal map used to track these messages is never cleaned, it will keep on growing.\nSince we're dealing with (short-lived) tests, that should be fine.","ref":"PhoenixTest.Playwright.Connection.html#received/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.start_link/1","doc":"","ref":"PhoenixTest.Playwright.Connection.html#start_link/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.subscribe/2","doc":"Subscribe to messages for a guid and its descendants.","ref":"PhoenixTest.Playwright.Connection.html#subscribe/2"},{"type":"module","title":"PhoenixTest.Playwright.Frame","doc":"Interact with a Playwright `Frame` (usually the \"main\" frame of a browser page).\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/frame.ts","ref":"PhoenixTest.Playwright.Frame.html"},{"type":"function","title":"PhoenixTest.Playwright.Frame.check/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#check/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.click/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#click/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.content/1","doc":"","ref":"PhoenixTest.Playwright.Frame.html#content/1"},{"type":"function","title":"PhoenixTest.Playwright.Frame.evaluate/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#evaluate/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.expect/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#expect/2"},{"type":"function","title":"PhoenixTest.Playwright.Frame.fill/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#fill/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.goto/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#goto/2"},{"type":"function","title":"PhoenixTest.Playwright.Frame.inner_html/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#inner_html/2"},{"type":"function","title":"PhoenixTest.Playwright.Frame.press/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#press/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.select_option/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#select_option/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.set_input_files/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#set_input_files/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.title/1","doc":"","ref":"PhoenixTest.Playwright.Frame.html#title/1"},{"type":"function","title":"PhoenixTest.Playwright.Frame.uncheck/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#uncheck/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.url/1","doc":"","ref":"PhoenixTest.Playwright.Frame.html#url/1"},{"type":"function","title":"PhoenixTest.Playwright.Frame.wait_for_selector/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#wait_for_selector/2"},{"type":"module","title":"PhoenixTest.Playwright.Message","doc":"Parse playwright messages.\nA single `Port` message can contain multiple Playwright messages and/or a fraction of a message.\nSuch a message fraction is stored in `bufffer` and continued in the next `Port` message.","ref":"PhoenixTest.Playwright.Message.html"},{"type":"function","title":"PhoenixTest.Playwright.Message.parse/4","doc":"","ref":"PhoenixTest.Playwright.Message.html#parse/4"},{"type":"module","title":"PhoenixTest.Playwright.Port","doc":"Start a Playwright node.js server and communicate with it via a `Port`.","ref":"PhoenixTest.Playwright.Port.html"},{"type":"function","title":"PhoenixTest.Playwright.Port.cli_path/1","doc":"","ref":"PhoenixTest.Playwright.Port.html#cli_path/1"},{"type":"function","title":"PhoenixTest.Playwright.Port.open/1","doc":"","ref":"PhoenixTest.Playwright.Port.html#open/1"},{"type":"function","title":"PhoenixTest.Playwright.Port.parse/2","doc":"","ref":"PhoenixTest.Playwright.Port.html#parse/2"},{"type":"function","title":"PhoenixTest.Playwright.Port.post/2","doc":"","ref":"PhoenixTest.Playwright.Port.html#post/2"},{"type":"module","title":"PhoenixTest.Playwright.Selector","doc":"Playright supports different types of locators: CSS, XPath, internal.\n\nThey can mixed and matched by chaining the together.\n\nAlso, you can register [custom selector engines](https://playwright.dev/docs/extensibility#custom-selector-engines)\nthat run right in the browser (Javascript).\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://playwright.dev/docs/other-locators\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/locator.ts\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/utils/isomorphic/locatorUtils.ts","ref":"PhoenixTest.Playwright.Selector.html"},{"type":"function","title":"PhoenixTest.Playwright.Selector.and/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#and/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.at/1","doc":"","ref":"PhoenixTest.Playwright.Selector.html#at/1"},{"type":"function","title":"PhoenixTest.Playwright.Selector.build/1","doc":"","ref":"PhoenixTest.Playwright.Selector.html#build/1"},{"type":"function","title":"PhoenixTest.Playwright.Selector.button/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#button/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.concat/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#concat/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.css/1","doc":"","ref":"PhoenixTest.Playwright.Selector.html#css/1"},{"type":"function","title":"PhoenixTest.Playwright.Selector.label/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#label/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.link/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#link/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.menuitem/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#menuitem/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.none/0","doc":"","ref":"PhoenixTest.Playwright.Selector.html#none/0"},{"type":"function","title":"PhoenixTest.Playwright.Selector.role/3","doc":"","ref":"PhoenixTest.Playwright.Selector.html#role/3"},{"type":"function","title":"PhoenixTest.Playwright.Selector.text/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#text/2"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}